(* Better animations

Adding an extra Y-offset to the parameters for movies.
The Y-offset can be positive or negative. 
When the offset is zero the figure is placed on the floor. 
When negative the picture shifts upwards, when positive
downward, like this:

PLAY>   ( a x-step y-step -- )
*)

inside also  forth definitions
: FPS       ( fps -- )      \ Minimal 1 Frame Per Second
    dm 1000 swap /  400 umin to wait ;

: .FPS      ( -- )      dm 1000 wait / . ;

inside definitions
\ Maximum movie length is 64 frames, it may be adjusted!
40 constant #ML

: .ANI      ( a -- )        \ Print one movie frame
    y >r  get-pict          \ Save Y, get picture data
    lines 0 ?do .bitrow loop \ Print one movie frame
    r> to y  wait ms ;      \ Restore Y

value OFFSET    \ Hold early stop offset
: LIMIT     ( xe0 -- xe1 )  offset xs 0 > ?negate + ; \ Adjust XE value using OFFSET

forth definitions   \ Set early stop X-offset
: >LIMIT    ( pix -- )      abs to offset ; 

\ : OFFSCR?       ( -- f )  \ Is X off display (lower then XE)
\    xs 0<  if  x xe <  exit  then
\    xs 0 > if  x xe >  exit  then  false ;
code OFFSCR?    ( -- f )
    tos sp -) mov
    #0 tos mov
    #0 adr xs & cmp  >? if,
        adr xe & adr x & cmp >? if,  #-1 tos mov  then,  next
    then,
    adr xs & tos cmp  >? if,
        adr x & adr xe & cmp  >? if,  #-1 tos mov  then,
    then,  
    next
end-code

inside definitions
: +FRAME    ( a1 -- a2 )        \ Go to next picture of a sequence
     get-pict  lines rows *  'pic + ;

value #O
: +FRAMES   ( a1 +n -- a2 )     \ Go to +nth picture frame of sequence a1
    0 TO #O  for
        dup @lines if  +frame  else  incr #O  then
    next ;

: PREPARE   ( -- )      \ Set movie start & end position
    xs 0= if            \ No movement?
        80 'pic @rows - 2/  to xp \ Yes, place picture in center
        8 'pic @lines - 2/ to yp
        xp to xe  exit
    then
    7F  'pic @rows negate   \ Start & end parameters
    xs 0< if swap then      \ Exchange when movement is right to left?
    to xp  limit  to xe     \ Set start & end
    8 'pic @lines - to yp ; \ Figure on floor

value XR  value X?          \ For specific needs to force the X start position
: START     ( a step -- )   \ Calculate & set all relevant movie parameters
    to xs  0 to yp  to 'pic  prepare
    x? if  xr  else  xp  then  yp xy ;

: SEQUENCE  ( -- )          \ Play an image sequence once
    begin
        'pic .ani           \ Show one image frame
        offscr?  xs +to x   \ To next X-position
    'pic @lines 0= or until ; \ End of image sequence, ready

forth definitions
\ Start movie A at frame SKIP
: >START    ( a skip -- )   +frames  to 'pic ;

\ Move X to an alternative start position
: ALT-X     ( x -- )    to xr  true to x? ;

: FRAME     ( a -- )        \ Animation test, frame by frame
    to 'pic  &page  0 to xs  prepare  0
    begin
        dup . 1+  'pic .demo
        begin  key bl = until
    'pic @lines 0= until  drop ;

: FINISH    ( -- )          \ Repeat movie until offscreen!
    xs if                   \ Picture moving?
        'pic  begin         \ Yes
            sequence  dup to 'pic
        offscr? until  drop
    then ;


value REP \ Hold REPetion value for current movie
\ Note that the end position maybe beyond the end of the movie
\ The software stops when a frame sequence ends
: PLAY>>    ( a st stp d #n - ) \ Play part of movie forward +n times
    to rep  >r >r >r r@ +frames \ a
    2r> -  swap r> start  'pic  \ +n a
    rep for
        over 0 ?do
            'pic 1 x@ 0= if leave then  'pic .ani
            offscr? if  leave  then  xs +to x
        loop
        dup to 'pic
    next  2drop  false to x? ;

\ Note that the end position maybe beyond the end of the movie
\ The software adjusts the length to the last frame of the given sequence
: <<PLAY    ( a st stp d #n - ) \ Play part of movie backward +n times
    to rep  >r >r           \ a st     Save D & STP
    2dup +frames drop       \ a st     Determine start point
    #O -  dup r> -          \ a st +n  Correct start point
    2 pick  r> start        \ a st +n  Set movie boundaries
    rep for                 \ Play movie +N times
            >r 2dup  r@ 0 ?do
                2dup 1- +frames .ani  1-
                offscr? if  leave  then  xs +to x
            loop  2drop  r>
    next  2drop  to 'pic  false to x? ;

\ Play movie A forward or backward one time, from frame ST to STP moving in direction D
: PLAY>     ( a st stp d - )    1 PLAY>> ;
: <PLAY     ( a st stp d - )    1 <<PLAY ;

\ Simple endless movie players
: MOVIE     ( a d -- )      \ Play movie forward endlessly
    begin  2dup >r  0 #ML r> play>  finish  key? until  2drop ;

: SETUP-PLAYER ( -- )   \ Initialise noForth OLED movie player at 15 fps
    setup-display  &page  false to x?
    setup-random  0F fps  0 >limit ;

xempty              \ Initialise FRAM page-1
' setup-player  to app
fresh definitions
shield PLAYER\  freeze

\ End
