(* Read & display & animate picture bitmaps

Picture & movie format:
    | nr of lines | pixel rows | Picture bitmap |
    | nr of lines | pixel rows | Picture bitmap |
    | nr of lines | pixel rows | Picture bitmap |
    | 0 |

*)

inside also  definitions
value LINES  value ROWS
value 'PIC   value 'DPIC
Value XP    Value YP
Value XS    Value XE        \ XS = Step Size, XE = End Position
\ XS will also be used to erase the number of rows a picture is replaced on the OLED


value RND   \ Work location for pseudo random numbers
decimal     \ Make pseudo random number ud
: RANDOM        ( -- ud )       rnd 31421 *  6927 +  dup to rnd ;
forth definitions
: CHOOSE        ( u1 -- u2 )    random  um* nip ;
: SETUP-RANDOM  ( -- )          31414 to rnd ;
hex

inside definitions
\ rows = Number of pixel rows to print, pa = Picture Address
\ : OL-FITS?  ( -- rows pa tf | ff )
\    y 8 u<  dup 0= ?exit  drop
\    x 0< if
\        x rows + dup 0 > if  80 umin else  drop  false exit  then
\        'pic x -  0 to x  true  exit
\    then
\    x 80 u< if
\        80 x - rows umin  'pic  true exit
\    then
\    false ;
code OL-FITS?   ( -- rows pa tf | ff )
    tos sp -) mov                   \ Make room for flag
    7 # day mov
    adr y &  day cmp  u>? if,       \ Y offscreen?
        #0 tos mov  next            \ Yes, leave false
    then,
    #0 adr x & cmp   >? if,         \ No, is X negative?
        #0 day mov
        adr x & sun mov             \ Yes, get x
        adr rows & sun add          \ Calc rows left
        sun day cmp  >? if,         \ Rows left to print?
            80 # day mov            \ Limit to 80 rows
            sun day cmp  u>? if,    \ More?
                day sun mov         \ Cut to 80 in SUN
            then,
            sun sp -) mov           \ Push rows
            adr 'pic & day mov      \ Picture addr. (pa) in DAY
            adr x & day sub         \ Add X to DAY
            day sp -) mov           \ Push pa
            #0 adr x & mov          \ To start of line
            #-1 tos mov  next
        then,
        #0 tos mov  next            \ Nothing left to show
    then,
    80 # adr x & cmp  u>? if,       \ X on screen?
        80 # sun mov
        adr x & sun sub
        adr rows & day mov          \ Limit to ROWS
        sun day cmp  u>? if,        \ More?
            day sun mov             \ Cut to ROWS in SUN
        then,
        sun sp -) mov               \ Push rows
        adr 'pic & sp -) mov        \ Push pa
        #-1 tos mov  next
    then,
    #0 tos mov  next
end-code

\ : NEXT-LINE     ( -- )      rows +to 'pic  incr y ;
code NEXT-LINE  ( -- )      \ To next display & picture line
    adr rows &  adr 'pic & add
    #1 adr y & add  next
end-code

: BLANK)    ( rows -- )
    40 {ol  for
        onscr? if  0 i2out  then  incr x
    next  i2stop} ;
\ code BLANK) ( rows -- )     \ Print rows white space
\   #8 202 & .b bic         \ CS active
\   10 # 202 & .b bis       \ OLED data
\   begin,
\   #0 tos cmp  pos? while,
\       80 # adr x & cmp  u>? if,
\           #0 day .b mov   \ Empty data row
\           adr inv? & day .b bix \ Invert pixelrow?
\           day 68E & .b mov \ Send pixel row
\           begin,  #1 688 & bit  cc? until,
\       then,
\       #1 adr x & add      \ x+1
\       #1 tos sub          \ Decrease rows
\   repeat,
\   sp )+ tos mov           \ Drop tos
\   #8 202 & .b bis  next   \ CS inactive
\ end-code

: ?BLANK    ( f -- )    if  xs ?rows  xy)  blank)  then ;

: @LINES     ( a -- lines )   1 f@ ;
\ code @LINES   ( a -- lines )
\    chere >r   tos sp -) mov  
\    #1 tos mov
\    ' x@ >body # pc mov
\ end-code

: @ROWS     ( a -- rows )     cell+ 1 f@ ;
\ code @ROWS  ( a -- rows )
\    #2 tos add  r> jmp
\ end-code

inside definitions
: GET-PICT  ( a -- )
    1 f@+ to lines  f@+ to rows  drop to 'pic ;
\ code XGET-PICT   ( a -- )
\   tos )+  adr lines & mov \ Set Y-size
\   tos )+  adr rows & mov  \ and X-size
\ ) tos ) adr 'pic & mov    \ Finally start of image
\   tos )+ adr 'pic & mov   \ Low word in 'pic
\   tos ) xx mov            \ High word in XX
\    sp )+ tos mov  next
\ end-code

: .ROWS    ( rows a -- )
    40 {ol  1  rot for  fc@+ i2out  incr x  next  2drop  i2stop} ;
\ Reg: W MOON TOS 
\ code .ROWS  ( rows a -- )
\    #8 202 & .b bic         \ CS active
\    10 # 202 & .b bis       \ OLED data
\    sp )+ moon mov          \ Rows to print
\    tos w mov               \ Low pict. address in W
\    begin,
\  )     #1 push  w push     \ High & low address part
\       xx push  w push     \ High & low address part
\        11A ,               \ rp )+ sun mova
\        1840 , 4A67 ,       \ sun ) tos .b x mov  ( Get bitrow )
\        adr inv? & tos .b bix \ Invert pixelrow?
\        tos 68E & .b mov    \ Send pixel row
\        begin,  #1 688 & bit  cc? until,
\        #1 w add            \ To next bitrow
\       0=? if,  #1 xx add  then, \ Correct high addr. part
\        #1 adr x & add      \ x+1
\        #-1 moon add        \ Decrease rows
\    0=? until,
\    sp )+ tos mov
\    #8 202 & .b bis  next   \ CS inactive
\ end-code

: .BITROW   ( -- )
    x >r  xs 0 > ?blank     \ White space on right?
    ol-fits? if  xy) .rows  else  rows +to x  then
    xs 0< ?blank            \ White space on left?
    r> to x  next-line ;

forth definitions
: .PIC      ( a x -- )    0 xy  get-pict  lines for  .bitrow  next ;

: .DEMO     ( a -- )
    dup @rows 2/ negate 40 +  \ Calc. center
    &page  0 to xs  .pic ;

: <SHIFT    ( a step -- )   \ Shift picture to left
    abs  negate to xs  dup @rows to rows
    rows negate xs + 80 do
        dup i .pic  wait ms
    xs +loop  drop ;

: SHIFT>    ( a step -- )   \ Shift picture to right
    abs to xs  80 xs + over @rows  negate do
        dup i .pic  wait ms
    xs +loop  drop ;

inside definitions
: -BYTE     ( "ccc" -- b )
    0.  bl-word count   \ Get next data byte
    2dup upper          \ Make string uppercase
    >number             \ Convert to number
    2drop drop ;        \ Leave lowest cell only

forth definitions
\ Compile pictures and picture frames for animation
: PIC:      ( xp yp "ccc" -- ) \ Read a bitmap
    8 / dup f,  over f,  *  
    for  -byte fc,  next ;  \ Read +n bytes bitmap & store in high mem.

: PIC;      ( -- )              0 f, ;
: PIC       ( "name" -- a )     fhere drop  constant ;

: .FREE     ( -- )
    ." FRAM: "  ivecs chere - u.
    ."  SRAM: " tib here - u.
    ." Flash: " .dfree ;

\ vocabulary PICS     \ Keep pictures together
fresh definitions
shield PIC\  freeze

\ End
