(* For noForth FR59xx, hardware SPI on MSP430FR5949 using port-2.

  Connect the SPI lines of eUSCI-B0 to an OLED screen.

  SPI master & slave

                     MSP430FR5994
                  -----------------
              /|\|              XIN|-
               | |                 |
               --|RST          XOUT|-
                 |                 |
           RST <-|P1.5         P5.0|-> Data Out (UCB1SIMO)
                 |                 |
            DC <-|P1.4             | 
                 |                 |
            CS <-|P1.3         P5.2|-> Serial Clock Out (UCB1CLK)

         in  out  dir  ren  sel0 sel1 
P1      200  202  204  206  20A  20C
P5      240  242  244  246  24A  24C

        UCB0CTLW  UCB0BRW  UCB0IFG  UCB0TXBUF  UCB0RXBUF
UCB0    640       646      66C      64E        64C
UCB1    680       686      6AC      68E        68C

For OLED ccreens the clock is max. 10 MHz for SSD1306 or 4 MHz for the SH1106
Only three lines need adjustement, all these lines marked: SH1106

*)

hex  inside also definitions
\ code ZERO?  ( u1 x -- u2 x f | x t )  \ LOOP replacement counter
\    #1 sp ) sub  0=? if,
\        tos sp ) mov  #-1 tos mov  next
\    then,
\    tos sp -) mov  #0 tos mov  next
\ end-code
code ZERO?      ( u1 x -- u2 x f | x t ) \ LOOP replacement counter
    8394 ,  0 ,  2004 ,  4784 ,  0 ,  4337 ,  next
    8324 ,  4784 ,  0 ,  4307 ,  next
end-code

\ Graphic OLED screen 128x64 pixels
value INV?                \ Inverted display?
code INV    ( b1 -- b2 )  \ Invert b1 to leave b2
    adr inv? & tos .b bix  next
end-code
\ code INV    ( b1 -- b2 )  \ Invert b1 to leave b2
\    E257 ,  adr inv? ,  next
\ end-code


\ Master SPI routine

inside also
: SPI-ON        ( -- )      07 24C *bic ; \ P5SEL1  3-wire SPI
: OLED-SETUP    ( -- )
    01 680 **bis    \ UCA1CTLW Reset eUSCI
    03 202 c!       \ P1OUT     Leds on
    03 204 c!       \ P1DIR
    07 24A *bis     \ P5SEL0    P2.4 P2.5 P2.6 is SPI
    spi-on          \           3-wire SPI
    38 204 *bis     \ P1DIR     Init CS, DC & RES
    18 202 *bis     \ P1OUT     CS & DC = 1
    E981 680 !      \ UCB0CTLW  Clk=High, MSB first, Master, Synch, 3-wire, SMClk
    04 686 !        \ UCB0BRW   Clock is 16Mhz/4 = 4 MHz
    01 680 **bic    \ UCB0CTLW  Free eUSCI
    03 202 *bic ;   \ P1OUT     Leds off

(* code SPI-I/O    ( b1 -- b2 )            \ Read and write at SPI-bus
    47C2 , 68E ,        \ tos 68E & .b mov      \ UCB1TXBUF
    B392 , 6AC ,        \ begin,  #1 688 & bit  \ UCB1STAT  Busy?
    2FFD ,              \ cs? until,
    4257 , 68C ,        \ 68C & tos .b mov      \ UCB1RXBUF
    next
end-code
: >SPI      ( b -- )    spi-i/o drop ;
*)

\ : >SPI   ( b -- )
\    68E c!  begin  01 688 bit* 0= until ;  \ UCB1STAT  Busy?
code >SPI  ( b -- )     \ Write to SPI-bus
    tos 68E & .b mov    \ UCB1TXBUF
    begin,  #1 688 & bit  cc? until, \ UCB1STAT  Busy?
    sp )+ tos mov  next
\   47C2 , 68E ,        \ UCB1TXBUF
\   B392 , 688 ,        \ UCB1STAT  Busy?
\   2FFD , 4437 ,  next
end-code


code COMM   ( -- )      10 # 202 & .b bic  next  end-code
code DATA   ( -- )      10 # 202 & .b bis  next  end-code
code {{     ( -- )      #8 202 & .b bic  next  end-code
code }}     ( -- )      #8 202 & .b bis  next  end-code
code WHITE  ( -- )      #0   adr inv? & mov  next  end-code
code BLACK  ( -- )      #-1  adr inv? & mov  next  end-code
\ code COMM   ( -- )      C0F2 ,  10 ,  202 ,  next  end-code \ OLED command
\ code DATA   ( -- )      D0F2 ,  10 ,  202 ,  next  end-code \ OLED screen data
\ code {{     ( -- )      C2F2 ,  202 ,  next  end-code \ OLED CS active
\ code }}     ( -- )      D2F2 ,  202 ,  next  end-code \ OLED CS inactive
\ code WHITE  ( -- )      4382 ,  adr inv? ,  next  end-code \ White on black display
\ code BLACK  ( -- )      43B2 ,  adr inv? ,  next  end-code \ Black in white display


\ OLED primitives

: {CMD      ( b -- )        {{  comm  >spi ;        \ Start OLED command stream
: {DATA     ( -- )          {{  data ;              \ Start OLED data stream
: >DATA     ( b -- )        inv >spi ;              \ Send one OLED data byte
: OL}       ( b -- )        >spi }} ;               \ End an OLED stream
: >BRIGHT   ( b -- )        81 {cmd ol} ;           \ b = 0 to 255 (max. brightness)
: ON/OFF    ( flag -- )     1 and  AE or {cmd }} ;  \ Display on/off
: INVERSE   ( flag -- )     1 and  A6 or {cmd }} ;  \ Display black or white

forth definitions
value X  value Y    \ Flag is true when X & Y are on screen
value WAIT          \ Picture hold time
inside definitions
code ONSCR?     ( -- fl )   \ X and Y on existing screen location, leave true
    8324 ,  4784 ,  0 ,     \ tos sp -) mov
    90B2 ,  80 ,  adr x ,   \ 80 # adr x & cmp
    2C05 ,  92B2 ,  adr y , \ cc? if,   #8 adr y & cmp
    2C02 ,  4337 ,  next    \   cc? if,   #-1 tos mov  next  then,
    4307 ,  next            \ then, #0 tos mov  next
end-code

\ : XY)         ( -- )
\    onscr? if               \ X & Y are valid?
\        y B0 or {cmd        \ Yes, set page
\        x 2 + dup 0F and >spi \ & set column
\        4 rshift 10 or ol}
\    then ;
\ : XY           ( x y )      to y  to x  xy) ;
\ Reg: DAY SUN  Note: this is a subroutine version of XY)
routine (XY)   ( -- )
    80 # adr x & cmp  u>? if,   \ X within 0 to 7F
        #8 adr y & cmp  u>? if, \ Y within 0 to 7
            #8 202 & .b bic     \ P1OUT  OLED CS active
            10 # 202 & .b bic   \ P1OUT  OLED command
            adr y & day mov     \ Y to DAY
            B0 # day bis
            day 68E & .b mov    \ UCB1TXBUF  Send
            begin,  #1 688 & bit  cc? until, \ UCB1STAT
            adr x & day mov     \ X in DAY
( SH1106 )  #2 day add          \ Only for SH1106
            day sun mov         \ Copy of X in SUN
            0F # day bia        \ X Low nibble
            day 68E & .b mov    \ UCB1TXBUF  Send
            begin,  #1 688 & bit  cc? until, \ UCB1STAT
            sun rra  sun rra  sun rra  sun rra \ sun/10
            0F # sun bia        \ X high nibble
            10 # sun bis
            sun 68E & .b mov    \ UCB1TXBUF  Send
            begin,  #1 688 & bit  cc? until, \ UCB1STAT
            #8 202 & .b bis     \ P1OUT  OLED CS inactive
        then,
    then,
    rp )+ pc mov  ( ret )
end-code
forth definitions
code XY     ( x y -- )      \ Set OLED column and row
    sp )+ adr x & mov
    tos adr y & mov
    sp )+ tos mov
chere >r
    (xy) # call
    next
end-code
code XY)    ( -- )      r> jmp  end-code

: SETUP-DISPLAY ( -- )
    oled-setup              \ Init. 4MHz USCI SPI
    20 202 *bic  1 ms  20 202 *bis \ Reset OLED
    false on/off            \ Display off
    0D5 {cmd  080 >spi      \ Set oscillator clock
    0A8 >spi  03F >spi      \ Set multiplexer ratio
    0D3 >spi  000 >spi      \ Display offset = 0
    040 >spi                \ Display starts at line 0
\   08D >spi  014 >spi      \ Charge pump on (not for SH1106)
\   020 >spi  002 >spi      \ Horizontal display mode (not for SH1106)
    0A1 >spi                \ Mirror X-axis (segment remap)
    0C8 >spi                \ Mirror Y-axis (COM scan dirction)
    0DA >spi  012 >spi      \ Alternate Com pin map  optional: 012 -> 02
    0D9 >spi  0F1 >spi      \ Set precharge cycles to high cap: F1 was 022
    0DB >spi  040 >spi      \ VCOMH voltage to max: 40 mwas 30
    0A4 >spi  0E3 ol}       \ Enable rendering from GDRAM, end stream
    80 >bright              \ Set brightness to 50%
    false inverse  white    \ Oled in normal mode
    true on/off 5 ms ;      \ Display on

\ : ?ROWS     ( n -- rows ) \ Leave nr. of rows left on screen
\    abs  begin  dup onscr? 0= and  while 1-  incr x  repeat ;
inside definitions
code ?ROWS  ( n -- rows )
    #0 tos cmp >? if,       \ ABS, make positive
       #-1 tos bix  #1 tos add
    then,
    begin,
    #0 tos cmp  0<>? while, \ Rows not zero?
        7F # day mov
        adr x & day cmp  u>? while, \ X pointer off screen?
            #1 tos sub  #1 adr x & add \ Correct X & rows
    repeat,
        then,  next
end-code

code >ROW   ( b -- )        \ Output one row of pixel data
    #1 adr x & add          \ Increase X
    #8 202 & .b bic         \ P1OUT  OLED CS active
    10 # 202 & .b bis       \ P1OUT  OLED screen data
    tos 68E & .b mov        \ UCB1TXBUF
    begin,  #1 688 & bit  cc? until, \ UCB1STAT
    sp )+ tos mov
    #8 202 & .b bis  next   \ P1OUT  OLED CS inactive
end-code
code >PIX  ' >row 3 cells + jmp  end-code

\ TOS = Picture address
code .ROW   ( rows a -- )
    #8 202 & .b bic         \ CS active
    10 # 202 & .b bis       \ OLED data
    sp )+ sun mov           \ Rows to print
    begin,
        tos )+ day .b mov   \ Read data row
        adr inv? & day .b bix \ Invert pixelrow?
        day 68E & .b mov    \ Send pixel row
        begin,  #1 688 & bit  cc? until,
        #1 adr x & add      \ x+1
        #-1 sun add         \ Decrease rows
    0=? until,
    sp )+ tos mov
    #8 202 & .b bis  next   \ CS inactive
end-code

\ : &FILL         ( +n b -- ) \ Pattern 'b' to +n columns
\    begin
\        onscr? 0= if        \ Cursor offscreen?
\            0 y 1+ 7 and xy \ Yes, to next line
\        then
\        dup >row            \ Output pattern
\    zero? until  drop ;
forth definitions
code &FILL      ( +n b -- ) \ Pattern 'b' to +n columns
    sp )+ moon mov
    begin,
        7F # day mov
        adr x & day cmp  u>? if,  \ X offscreen?
            #1 adr y & add
            #0 adr x & mov
            7 # day mov
            adr y &  day cmp  u>? if, \ Y offscreen?
                #0 adr y & mov
            then,
            (xy) # call
        then,
        #8 202 & .b bic         \ P1OUT  OLED CS active
        10 # 202 & .b bis       \ P1OUT  OLED screen data
        0 # tos bic             \ Noop
        tos 68E & .b mov        \ UCB1TXBUF
        begin,  #1 688 & bit  cc? until, \ UCB1STAT
        #1 adr x & add          \ Increase X
        #8 202 & .b bis         \ P1OUT  OLED CS inactive
        #1 moon sub
    0=? until,
    sp )+ tos mov
    next
end-code
inside definitions
: .BITROW   ( a +n -- )     \ Output half of +Nx14 (big) character
    0 >pix  for  count >pix 1+  next \ Output every second bit row
    drop  0 >pix ;

: EOL           ( +n -- )   for  0 >pix  next ;

value O-EMIT                \ OLED emit vector
forth definitions
: &ERASE        ( -- )          400 0 &fill ;   \ Erase screen  
: &PAGE         ( -- )          &erase 0 0 xy ; \ To upper left corner
: &CR           ( -- )          0 y 2 + xy ;    \ Valid for 16-bit chars!
: &EMIT         ( c -- )        o-emit execute ;
: &SPACE        ( -- )          bl &emit ;
: &SPACES       ( u -- )        for  &space  next ;
: &TYPE         ( a u -- )      for  count &emit  next drop ;
: &U.           ( n -- )        0 d.str &type &space ;

: &"            ( -- )          \ ." voor OLED
    flyer  postpone s"  postpone &type ;  immediate

: XY"       ( x y ccc -- )      \ XY" voor OLED
    flyer  postpone xy  postpone &" ; immediate

code C>N        ( c -- +n )     \ Convert char to bitmap index number
    9037 ,  80 ,  2802 ,        \ 80 # tos cmp  cs? if,
    4037 ,  80 ,  8037 ,  20 ,  \ 80 # tos mov  then,  20 # tos sub
    next
end-code

fresh  ' setup-display  to app  \ Init. OLED at startup
shield SSD1306\  freeze

\ End
