forth: HFR-demo
\ First HI FRAM use for Micro Launchpad FR5

(* Hi part of FRAM

  This code will work on all MPU's with FRAM above 10000
  note that this code has a maximum address range of 10000 too.

*)

hex
: D-        ( dn1 dn2 -- dn3 )  dnegate d+ ;
: M+        ( d n -- d )        s>d d+ ;
: 2@        ( a -- lo hi )      dup cell+ @ swap @ ;

\ Cell wide read and store operators for FRAM
: XC@+      ( xa -- xa+ x )     2dup xc@ >r  1 m+  r> ;
: X@+       ( xa1 -- xa2 x )    2dup x@ >r   2 m+  r> ;
: X+!       ( n xa -- )         2>r  2r@ x@ +  2r> x! ;

\ First cell in hi FRAM is used as XHERE, this way it is always up to date
\ We have to take care manually of the forget action on this address pointer
\ Note that XHERE is initialised at address 10002 right behind itself!!
\ The out of X-memory error message, shows an error in XALLOT !!
: XDP       ( -- xa )           10000. ;  2 xdp x!  \ Define and initialise XHERE
: XHERE     ( -- xa )           xdp  2dup x@  m+ ;
: XFREE     ( -- u )            14000. xhere d- drop ; \ Correct for MSP430FR5949
: XALLOT    ( +n -- )           xfree  over u< ?abort  xdp x+! ;
: XC,       ( b -- )            xhere  1 xallot  xc! ;
: X,        ( x -- )            xhere  2 xallot  x! ;
: XCREATE   ( -- xa )           create  xhere , ,  does> 2@ ;

: DU.R      ( xa +n -- )        >r du.str  r> rtype ;
: XDMP          ( xa -- )
    hex  begin
        cr  2dup 5 du.r ." : "
        10 0 do  2dup i m+ xc@ 2 .r space  loop  ch | emit \ Show hex
        10 0 do  2dup i m+ xc@ pchar emit  loop  10 m+      \ Show Ascii
    key bl <> until  2drop ;

shield XRAM\  freeze


\ Startup using XRAM
2 xdp x!    \ Restore XRAM pointer

\ Store string a u in XRAM and type out when executed
: XX$           ( a u -- )
    create
        xhere , ,
        ?dup if
            dup xc,
            0 ?do  count xc,  loop
        then  drop
    does>
        2 322 *bix          \ Toggle led
        2@  xc@+ 0 ?do  xc@+ emit  loop  2drop ;

s" Hi Forth-gg members, here is the Micro Launchpad FR5"    xx$ .FORTHGG
s" Preloaded with FRAM words for high FRAM data space"      xx$ .FRAM

: START     ( -- )          \ Type alternative startup strings
    2 324 *bis              \ Init led output
    4 0 do
        cr .forthgg  200 ms
        cr .fram     200 ms
        cr
    loop
    2 322 *bic              \ Led off
    10000. xdmp ;           \ Show XRAM contents

' start     to app
shield START\  freeze

end-of-file

\ End ;;;
