forth: timer-3
(* Large counter to time lectures etc. using graphic char set. Size: 2266 bytes

                     MSP430FR5994
                  -----------------
              /|\|              XIN|-
               | |                 |
               --|RST          XOUT|-
                 |                 |
                 |             PJ.1|-> XT1CLK/16384 = 1 Hz
                 |                 |
                 |                 | 
                 |                 |


- Port - in  out  dir  ren  sel0 sel1 selc  - CS registers  - Spec. Funct. Reg.
P1      200  202  204  206  20A  20C  216       0 - 160     SFRIE1  100
P2      201  203  205  207  20B  20D  217       1 - 162     SFRIFG1 102
P3      220  222  224  226  22A  22C  236       2 - 164     SFRRPCR 104
P4      221  223  225  227  22A  22C  237       3 - 166
PJ      320  322  324  326  32A  32C  336       4 - 168
                                                5 - 16A
                                                6 - 16C
TA1 timer registers:
TA1CTL   - 380
TA1CCTL0 - 382
TA1CCR0  - 392

*)

v: inside also  definitions
code INT-ON     D232 ,  next  end-code
code INT-OFF    C232 ,  next  end-code

value SECS          \ Seconds
value MINS          \ Minutes
value HOURS         \ Hours
value DAYS          \ Day counter

\ Set ACLK to 32 kHz, SFRIFG1 contains the OFIFG flag
: 32KHZ     ( -- )
    int-off  30 32A **bis   \ PJSEL0  LFXT osc. on
    A5 161 c!               \ CSCTL0H Open clock access
    0700 164 **bic          \ CSCTL2  ACLK = LFXTCLK
    0700 166 **bic          \ CSCTL3  ACLK/1
    0D08 0168 !             \ CSCTL4  32kHz drive strength lower
    begin                   \ Test 32kHz oscillator
        1 16A **bic         \ CSCTL5  Clear LFXT fault flag
        2 102 **bic         \ SFRIFG1 Clear XT1 fault flag
    2 102 bit** 0= until    \ SFRIFG1 Ready when XT1 runs fine!
    0 161 c! ;              \ CSCTL0H Close clock access


\ Set timer-A1 compare interrupt on with ACLK and init. leds
\ Clock divider=/1, Up mode & Timer A1 clear.
: CLOCK-ON      ( -- )
    32khz                   \ 32kHz on
    2 324 *bis  2 322 *bic  \ PJDIR, PJOUT  Led off
    4000 392 !              \ TA1CCR0  Half second tick
    114 380 !               \ TA1CTL   Set timer mode to ACLK/1
    10 0382 **bis  int-on ; \ TA1CCTL0 Enable interrupts on Compare

: SET-CLOCK     ( hours minutes -- )
    0 to secs  to mins  to hours ;

: GET-CLOCK     ( -- seconds minutes hours )
    secs  mins  hours ;

\ 24 hour clock interrupt subroutine
routine CLOCK-INT   ( -- )
    #2 322 & .b bix         \ PJOUT   Toggle LED
    #2 322 & .b bit cc? if, \ Second past?
        #1 adr secs & add   \ Increase seconds
        dm 60 # adr secs & cmp =? if,       \ Minute done?
\ Weekdays Real Time Clock
            #0 adr secs & mov
            #1 adr mins & add
            dm 60 # adr mins & cmp =? if,   \ Hour done?
                #0 adr mins & mov
                #1 adr hours & add
                dm 24 # adr hours & cmp =? if,  \ Day done?
                    #0 adr hours & mov
                    #1 adr days & add
                    7 # adr days & cmp =? if,   \ Week done?
                         #0 adr days & mov
                    then,
                then,
            then,
        then,
    then,
    reti
end-code

clock-int  FFE2 vec!        \ Timer-A1 TA1CCR0 vector

\ Sleek large characters 14 x 32 by Willem Ouwerkerk (code space: .... bytes)
chere
: |||   ( bitrow -- )
    0D parse drop  0  ( a x )
    10 0 ?do
        2*  over c@ ch X =  -
        >r 1+ r>
    loop  >r
    0  10 0 ?do
        2*  over c@ ch X =  -
        >r 1+ r>
    loop  ,  r> ,  drop ;

create LARGE1    \ Large characters 14 x 32 pixels
||| ................................
||| ................................
||| ................................
||| ................................
||| ................................
||| ................................
||| ................................
||| ................................
||| ................................
||| ................................
||| ................................
||| ................................
||| ................................
||| ................................

||| .........XXXXXXXXXXXXXXXX.......
||| ......XXXXXXXXXXXXXXXXXXXXXX....
||| ....XXX....................XXX..
||| ...XXX......................XXX.
||| ...XX........XX..............XX.
||| ..XX..........XX..............XX
||| ..XX...........XX.............XX
||| ..XX............XX............XX
||| ..XX.............XX...........XX
||| ...XX.............XX.........XX.
||| ...XXX......................XXX.
||| ....XXX...................XXX...
||| ......XXXXXXXXXXXXXXXXXXXXX.....
||| .........XXXXXXXXXXXXXXX........

||| ................................
||| ..X.............................
||| ..X.......................X.....
||| ..XX......................XX....
||| ..XX.......................XX...
||| ..XX........................XX..
||| ..XXXXXXXXXXXXXXXXXXXXXXXXXXXXX.
||| ..XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
||| ..XX............................
||| ..XX............................
||| ..XX............................
||| ..X.............................
||| ..X.............................
||| ................................

||| ..XXXXXXX.................X.....
||| ..XXXXXXXXX................XX...
||| ..XX......XX................XX..
||| ..XX.......XX................XX.
||| ..XX........XX................X.
||| ..XX.........XX...............XX
||| ..XX..........XX..............XX
||| ..XX...........XX.............XX
||| ..XX............XX............XX
||| ..XX.............XX...........X.
||| ..XX..............XX.........XX.
||| ..XX...............XXX......XX..
||| ..X.................XXXXXXXXX...
||| ..X...................XXXXX.....

||| .......XX................XX.....
||| .....XX....................XX...
||| ....XX......................XX..
||| ...XX........................XX.
||| ...X..........................X.
||| ..XX..........................XX
||| ..XX.............X............XX
||| ..XX............XX............XX
||| ..XX............XX............XX
||| ...X...........XXXX...........X.
||| ...XX.........XX..XX.........XX.
||| ....XXX......XXX...XXX.....XXX..
||| .....XXXXXXXXXX.....XXXXXXXXX...
||| .......XXXXXX.........XXXXX.....


||| ..........XXX...................
||| ..........XXXXX.................
||| ..........XX..XXX...............
||| ..........XX...XXXX.............
||| ..........XX.....XXXXX..........
||| ..........XX........XXXXX.......
||| ..........XX...........XXXXXXX..
||| ..........XX..............XXXXXX
||| ..........XX....................
||| ..........XX....................
||| ..XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
||| ..XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
||| ...........X....................
||| ...........X....................

||| .......X............XXXXXXXXXXXX
||| .....XX.............XXXXXXXXXXXX
||| ....XX..............XX........XX
||| ...XX...............XX........XX
||| ..XX................XX........XX
||| ..XX................XX........XX
||| ..XX................XX........XX
||| ..XX................XX........XX
||| ..XX...............XX.........XX
||| ...XX..............XX.........XX
||| ....XX............XX..........XX
||| .....XXX........XXX...........XX
||| ......XXXXXXXXXXX..............X
||| .........XXXXXX................X

||| .........XXXXXXXXXXXXXXXX.......
||| .......XXXXXXXXXXXXXXXXXXXX.....
||| .....XXX.................XXXX...
||| ....XX......................XX..
||| ...XX........................XX.
||| ..XX..............X...........X.
||| ..XX..............X...........XX
||| ..XX.............XX...........XX
||| ..XX.............XX...........XX
||| ..XX.............XX...........XX
||| ...XX...........XX.............X
||| ....XXX.......XXX..............X
||| ......XXXXXXXXXX................
||| ........XXXXX...................

||| ..XXXX.........................X
||| ...XXXXX.......................X
||| ......XXXX....................XX
||| ........XXXX................ .XX
||| ..........XXXX................XX
||| ............XXXX..............XX
||| ..............XXXX............XX
||| ................XXXX..........XX
||| ..................XXXX........XX
||| ....................XXXX......XX
||| ......................XXXX....XX
||| ........................XXXX..XX
||| ..........................XXXXXX
||| ............................XXXX

||| .......XXXXXX..........XXXX.....
||| .....XXXXXXXXXX......XXXXXXXX...
||| ....XX........XX....XX......XX..
||| ...XX..........XX..XX........XX.
||| ...X.............XX...........X.
||| ..XX.............XX...........XX
||| ..XX.............XX...........XX
||| ..XX.............XX...........XX
||| ..XX.............XX...........XX
||| ...X.............XX...........X.
||| ...XX..........XX..XX........XX.
||| ....XX........XX....XX......XX..
||| .....XXXXXXXXXX......XXXXXXXX...
||| .......XXXXXX..........XXXX.....

||| .....................XXXXX......
||| ...................XXXXXXXXX....
||| ..................XX.......XXX..
||| ..X..............XX......... XX.
||| ..X.............XX............X.
||| ..XX...........XX.............XX
||| ..XX...........XX.............XX
||| ...X...........XX.............XX
||| ...XX...........X.............XX
||| ....XX..........X.............X.
||| .....XX......................XX.
||| ......XXX...................XX..
||| ........XXXXXXXXXXXXXXXXXXXXX...
||| ...........XXXXXXXXXXXXXXX......

||| ................................
||| ................................
||| ................................
||| .......XX...........XX..........
||| ......XXXX.........XXXX.........
||| .....XX..XX.......XX..XX........
||| ....XX....XX.....XX....XX.......
||| ....XX....XX.....XX....XX.......
||| .....XX..XX.......XX..XX........
||| ......XXXX.........XXXX.........
||| .......XX...........XX..........
||| ................................
||| ................................
||| ................................

||| ................................
||| ................................
||| ................................
||| .X.....XX...........XX..........
||| .X....XXXX.........XXXX.........
||| .XX..XX..XX.......XX..XX........
||| ..XXXX....XX.....XX....XX.......
||| ...XXX....XX.....XX....XX.......
||| .....XX..XX.......XX..XX........
||| ... ..XXXX.........XXXX.........
||| .......XX...........XX..........
||| ................................
||| ................................
||| ................................

||| ................XX..............
||| ..............XX..XX............
||| .............XX....XX...........
||| ............XX......XX..........
||| ...........XX........XX.........
||| ..........XX..........XX........
||| .........XX............XX.......
||| ........XX..............XX......
||| .......XX................XX.....
||| ......XX..................XX....
||| .....XX....................XX...
||| ....XX......................XX..
||| ...XX........................XX.
||| ..XX..........................XX

||| ................................
||| ................................
||| ............XX.....XX...........
||| ............XX.....XX...........
||| ............XX.....XX...........
||| ............XX.....XX...........
||| ............XX.....XX...........
||| ............XX.....XX...........
||| ............XX.....XX...........
||| ............XX.....XX...........
||| ............XX.....XX...........
||| ............XX.....XX...........
||| ................................
||| ................................

||| ..XX..........................XX
||| ...XX........................XX.
||| ....XX......................XX..
||| .....XX....................XX...
||| ......XX..................XX....
||| .......XX................XX.....
||| ........XX..............XX......
||| .........XX............XX.......
||| ..........XX..........XX........
||| ...........XX........XX.........
||| ............XX......XX..........
||| .............XX....XX...........
||| ..............XX..XX............
||| ................XX..............

||| ..........................X.....
||| ...........................XX...
||| ............................XX..
||| .............................XX.
||| ..............................X.
||| ....XXX.......................XX
||| ..XXXXXXX....XXXXX............XX
||| ..XXXXXXX....XXXXXX...........XX
||| ....XXX...........XX..........XX
||| ...................XX.........X.
||| ....................XX.......XX.
||| .....................XX.....XX..
||| ......................XXXXXXX...
||| ........................XXX.....

||| .........XXXXXXXXXXXXXXXX.......
||| ......XXXXXXXXXXXXXXXXXXXXXX....
||| .....XX....................XX...
||| ....XX......................XX..
||| ...XX........................XX.
||| ...X..........................X.
||| ..XX..........XXXXXXX.........XX
||| ..XX........XXXXXXXXXXX.......XX
||| ..XX.......XX.........XX......XX
||| ..XX......XX..........XX......X.
||| ...X......XX.........XX......XX.
||| ...XX......XX.......X.......XX..
||| ....XX......XXXXXXXXXXXXXXXXX...
||| ......X....X..XXXXXXXXXXXX......

||| ..XXXXXXXXXXXXXXXXXXXXXX........
||| ..XXXXXXXXXXXXXXXXXXXXXXXXX.....
||| ..........................XXX...
||| ...........................XXX..
||| ............XX..............XXX.
||| ............XX...............XXX
||| ............XX...............XXX
||| ............XX...............XXX
||| ............XX...............XXX
||| ............XX..............XXX.
||| ...........................XXX..
||| ..........................XXX...
||| ..XXXXXXXXXXXXXXXXXXXXXXXXX.....
||| ..XXXXXXXXXXXXXXXXXXXXXX........

||| ..XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
||| ..XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
||| ..XX..........................XX
||| ..XX.............X............XX
||| ..XX.............X............XX
||| ..XX............XX............XX
||| ..XX............XX............XX
||| ..XX............XX............XX
||| ...X............XX............X.
||| ...XX..........X.XX..........XX.
||| ....XX........XX..XX........XX..
||| .....XX......XX....XX......XX...
||| ......XXXXXXXX......XXXXXXXX....
||| .........XX............XX.......

||| ..........XXXXXXXXXXXXXX........
||| .......XXXXXXXXXXXXXXXXXXXX.....
||| .....XX....................XX...
||| ....XX......................XX..
||| ...XX........................XX.
||| ...X..........................X.
||| ..XX..........................XX
||| ..XX..........................XX
||| ..XX..........................XX
||| ..XX..........................XX
||| ...X..........................X.
||| ...XX........................XX.
||| ....XX......................XX..
||| ......XX..................XX....

||| ..XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
||| ..XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
||| ..XX..........................XX
||| ..XX..........................XX
||| ..XX..........................XX
||| ..XX..........................XX
||| ..XX..........................XX
||| ...X..........................X.
||| ...XX........................XX.
||| ....XX......................XX..
||| .....XX....................XX...
||| ......XX..................XX....
||| ........XXXXXXXXXXXXXXXXXX......
||| ...........XXXXXXXXXXXX.........

||| XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
||| XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
||| XX............................XX
||| XX............................XX
||| XX.............XX.............XX
||| XX.............XX.............XX
||| XX.............XX.............XX
||| XX.............XX.............XX
||| XX..............X.............XX
||| XX..............X.............XX
||| XX............................XX
||| XX............................XX
||| X..............................X
||| X..............................X

||| XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
||| XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
||| ..............................XX
||| ..............................XX
||| ...............XX.............XX
||| ...............XX.............XX
||| ...............XX.............XX
||| ...............XX.............XX
||| ................X.............XX
||| ................X.............XX
||| ..............................XX
||| ..............................XX
||| ...............................X
||| ...............................X


: .LBITROW   ( a +n -- )     \ Output quarter of +Nx32 (large) character
    0 >row 0 >row
    for  count >row 3 +  next \ Output every third bit row
    drop  0 >row 0 >row ;

\ : EOL           ( +n -- )     for  0 pixelrow  next ;

: L&EMIT    ( c -- )
    bl -  dup 0 > if  0F -  then
    80 x -  dup 12 < if     \ Line full?
        dup eol             \ Erase to end Of Line
        x y 1+ xy  dup eol  \ Erase end of next lines too
        x y 1+ xy  dup eol
        x y 1+ xy  dup eol
        0 y 1+ xy           \ To start of new line
    then  drop
    38 * large1 +  dup 0E .lbitrow  \ First quarter of large char
    x y 1+ xy  1+  dup 0E .lbitrow  \ Second quarter of large char
    x y 1+ xy  1+  dup 0E .lbitrow  \ Third quarter
    x y 1+ xy  1+  0E .lbitrow      \ Fourth quarter
    x 12 + y 3 -  xy ;      \ x & y to next char position

v: extra definitions
: LARGE     ['] l&emit to o-emit ;


cr ." Length " chere swap - dm .  ." bytes "

v: inside definitions
: TIK       ( -- )
    bl  2 322 bit* if  drop  ch :  then  hold ;  \ Show second tick

: .KLOK     ( m h -- )
    0 <# # # #> &type
    0 <# # # tik #> &type ;

v: extra definitions
: KLOK      ( -- )          \ Lecture timer in decimal
    mount  setup-display  5 ms  &page  large
    clock-on  0 0 set-clock  decimal
    begin
        #Y 20 = if
            14 0 xy  mins hours .klok
        else
            14 1 xy  mins hours .klok
            14 5 xy  5A 1C &fill
        then
    stop? until  hex ;

: COUNTER   ( -- )          \ Hexadecimal counter demo
    mount  setup-display  5 ms  &page  large
    clock-on  hex  0
    begin
        #Y 20 = if
            14 0 xy  dup dup ><  .klok
        else
            14 1 xy  dup dup ><  .klok
            14 5 xy  5A 1C &fill
        then
        1+  30 ms
    stop? until  drop ;

v: fresh definitions
' KLOK  to app
shield KLOK\  freeze

\ End ;;;

